<?php
/**
 * View a person's record.
 * @package iHRIS
 * @subpackage DemoManagePage
 * @access public
 * @author Carl Leitner <litlfred@ibibilio.org>
 * @copyright Copyright &copy; 2006, 2008 IntraHealth International, Inc. 
 * @since Demo-v2.a
 * @version Demo-v2.a
 */


/**
 * The page class for displaying the a person's assignment history
 * @package iHRIS
 * @subpackage DemoManagePage
 * @access public
 */
class Botswana_AssignmentHistoryPage extends I2CE_Page {

    /**
     * Return the title for this page.
     * @return string
     */
    protected function getTitle() { return "Assignment History"; }


    /**
     * Load the HTML template files for editing and confirming the index and demographic information.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
    }
        
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        parent::action();
        if (!$this->hasPermission('task(person_can_view_assignment)')) {
            $this->userMessage("You do not have permission to view the work assignments");
            return false;
        }
        return $this->showAssignmentHistory();
    }
                


    protected function showAssignmentHistory() {
        $factory = I2CE_FormFactory::instance();
        $person = $factory->createContainer( $this->get('id') );
        $person->populate();
        $this->template->setForm( $person );
        $positionIDs = $person->getChildIds('person_position','-start_date'); //get the positions ordered by descending start date
        $postions = array();
        $this->template->setDisplayData( "history_header",  "Assignment History" );
        $firstPosition = true;
        foreach ($positionIDs as $positionId) {
            $position =  $factory->createContainer("person_position".'|'.$positionId);
            if (!$position instanceof iHRIS_PersonPosition) {
                continue;
            }
            $position->populate();
            if ($firstPosition) {
                $firstPosition = false; 
            } else {
                $this->template->appendFileById( "hr.html", "hr", "assignment_history" );
            }
            $positionNode = $this->template->appendFileById( "assignment_history_position.html", "div", "assignment_history" );
            $this->template->setForm($position,$positionNode);
            $assignmentListNode = $this->template->getElementById('assignment_list');
            if ($assignmentListNode instanceof DOMElement) {
                $assignmentListNode->setAttribute('id', "assignment_list_$positionId");
            }
            $assignmentIDs = $position->getChildIds('assignment','-start_date');
            foreach ($assignmentIDs as $assignmentId) {
                $assignment = $factory->createContainer('assignment'.'|'.$assignmentId);
                if (!$assignment instanceof I2CE_Form && $assignment->getName() != "assignment" ) {
                    continue;
                }
                $assignment->populate();
                $assignmentNode = $this->template->appendFileById( "assignment_history_assignment.html", "div", "assignment_list_$positionId" );
                $this->template->setForm( $assignment,$assignmentNode );
            }
            if (count($assignmentIDs) == 0 && $this->hasPermission('task(person_can_edit_assignment)'))  {
                $assignmentNode = $this->template->appendFileById( "assignment_history_add.html", "div", "assignment_list_$positionId" );
                $this->template->appendFileById( "hr.html", "hr", "assignment_history" );
            }
            $this->template->setForm( $position, $positionNode );

        }
        return true;
    }
}



# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
