<?php
/**
* © Copyright 2011 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package botswana
* @subpackage assignment
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.0.12
* @since v4.0.12
* @filesource 
*/ 
/** 
* Class Botswana_Module_Assignment
* 
* @access public
*/


class Botswana_Module_Assignment extends I2CE_Module {

    public static function getMethods() {
        return array(
            'iHRIS_PageView->action_person_position' => array(
                'method'=>'action_person_position',
                'priority'=> 349  //we need to override the action_person_postion in the iHRIS_PersonPostion module which has priority 350
                )
            );
    }


    /**
     * Find and return the most recent position held by this person.
     * 
     * @return I2CE_Form or null
     */
    public function getLastPosition($person) {
        if (!$person instanceof iHRIS_Person) {
            return null;
        }
        $where = array(
            'operator'=>'FIELD_LIMIT',
            'field'=>'end_date',
            'style'=>'null'
            );
        $per_pos_id = I2CE_FormStorage::search('person_position', $person->getNameId(),$where,'-start_date',1);
        if (!$per_pos_id) {
            return null;
        }
        $pos = I2CE_FormFactory::instance()->createContainer('person_position'.'|'.$per_pos_id);
        if (!$pos instanceof iHRIS_PersonPosition) {
            return null;
        }
        $pos->populate();
        return $pos;
    }




    public function action_person_position($page) {
        if (!$page instanceof iHRIS_PageView) {
            return false;
        }
        $person = $page->getPerson();
        if (!$person instanceof iHRIS_Person) {
            return false;
        }
        $template=$page->getTemplate();
        if (  ($lastPos = $this->getLastPosition($person)) instanceof iHRIS_PersonPosition) {
            $person->addChildForm($lastPos);
            $page->appendChildTemplate('person_position','siteContent');
        } else {
            $page->addLastChildForm('person_position','start_date', 'siteContent');
        }
        if (!array_key_exists('person_position',$person->children) || !is_array($person->children['person_position']) 
            || count($person->children['person_position']) != 1) {
            //echo count($person->children['person_position']);
            return true;
        }
        reset($person->children['person_position']);        
        $pers_pos = current($person->children['person_position']);        
        if ( !$pers_pos instanceof iHRIS_PersonPosition)  {
            return true;
        }

        $pers_pos->getField( "position" )->setHref( "view_position?id=" );

        $salary_node = $template->appendFileById( "view_salary.html", "div", "salary" );        
        $market_rate = 0;
        $sg =I2CE_FormFactory::instance()->createContainer($pers_pos->getSalaryGradeID()); 
        if ( $sg instanceof iHRIS_SalaryGrade) {
            $sg->populate(); 
            $template->setForm($sg); 
            $market_rate = $sg->getField('midpoint')->getValue(); 
            if (is_array($market_rate) && array_key_exists(1,$market_rate)) {
                $market_rate =$market_rate[1]; 
            } else {
                $market_rate = 0;
            }
        }
        //$pers_pos->populateLast(array("salary"=>"start_date"));        
        $salaryForm =$pers_pos->getLastSalary();
        if ($salaryForm instanceof iHRIS_Salary) {
            $template->setForm($salaryForm);
            $salary = $salaryForm->getField('salary')->getValue();  
            $salary =$salary[1];  
            if ($market_rate != 0) { 
                $percent = sprintf("%.1f%%",100*(float) $salary/ (float) $market_rate); 
            } else { 
                $percent = '0%'; 
            } 
            $template->setDisplayData("percent_salary",$percent);
        }
        $today = I2CE_Date::now()->getValues();
        $where =array(
            'operator'=>'OR',
            'operand'=>array(
                0=> array (
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'end_date',
                    'style'=>'null',
                    'data'=>array()
                    ),
                1=> array (
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'end_date',
                    'style'=>'greaterthan_equals',
                    'data'=>$today
                    ),
                )
            );
        //$pers_pos->populateChild('assignment', array(),$where);  there is a bug in 4.0.12 populateChild... reproducing intended results here.
        $ids = $pers_pos->getChildIds( 'assignment', array(), $where );
        if (!is_array($ids) || count($ids) == 0) {
            return true;
        }
        foreach( $ids as $id ) {
            $pers_pos->addChild('assignment',$id);
        }
        //end of populate child replcaement

        if (!array_key_exists('assignment',$pers_pos->children) || !is_array($pers_pos->children['assignment'])) {
            return true;
        }
        foreach ($pers_pos->children['assignment'] as $assign) {
            $assignment_node = $template->appendFileById( "view_assignment.html", "div", "assignment" );        
            if (!$assignment_node instanceof DOMNode) {
                continue;
            }
            $template->setForm($assign,$assignment_node);
        }
        return true;
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
