<?php
/*
 * © Copyright 2007, 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing assignment details to the database.
 * 
 * @package iHRIS
 * @subpackage Manage
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>/Carl Leitner<litlfred@ibiblio.org>
 * @copyright Copyright &copy; 2007, 2008 IntraHealth International, Inc. 
 * @since v2.0.0
 * @version v2.0.0
 */

/**
 * Page object to handle the adding or editing assignment details to the database.
 * 
 * @package iHRIS
 * @subpackage Manage
 * @access public
 */
class Botswana_PageFormAssignment extends I2CE_PageForm {
    /**
     * Return the default HTML file used by this page.
     * @return string
     */
    protected function getDefaultHTMLFile() { return "form_person_base.html"; }
        
    /**
     * Return the title for this page.
     * @return string
     */
    protected function getTitle() { return "Assignment Change"; }


    /**
     * @var integer The id of the previous assignment record
     */
    private $last_id;
    /**
     * @var I2CE_Form the {@link Botswana_Assignment} object for the previous record.
     */
    private $last;
    /**
     * Return the form name for this page.
     * @param boolean $html Set to true if this is to be used for the html template page.
     * @return string
     * @see PersonPageForm
     */
    protected function getForm( $html=false ) {
        return "assignment"; 
    }
    /**
     * @var integer The record id number of the object being edited.
     */
    protected $id;
    /**
     * @var integer The record id number of the parent person_position of the object being edited
     */
    protected $position_id;

    /**
     * @var integer The record id number of the person of the object being edited
     */
    protected $person_id;


    /**
     * @var Person person  The person of the object being edited
     */
    protected $person;

    /**
     * @var PersonPosition $personPosition The record id number of the person of the object being edited
     */
    protected $person_position;

    /**
     * @var Botswana_Assignment $assignement The assignment object
     */
    protected $assignment;


    /**
     * Create a new instance of this page.
     * 
     * This will call the parent constructor and then setup the base
     * template pages for the {@link Template template}.  It also sets up the values
     * for the member variables.
     */
    /**
     * Create a new instance of this page.
     * 
     * This will call the parent constructor and then setup the base
     * template pages for the {@link Template template}.  It also sets up the values
     * for the member variables.
     */
    public function __construct($args,$request_remainder) {
        parent::__construct($args,$request_remainder);
        $this->id = 0;
        if ( $this->isPost() && $this->post_exists( 'id' ) ) {
            $this->id = $this->post('id');
            $assignment = $this->factory->createContainer( $this->getForm() . '|'  . $this->id );            
        } elseif ( $this->get_exists( 'id' ) ) {
            $this->id = $this->get('id');
            $assignment = $this->factory->createContainer( $this->getForm() . '|'  . $this->id );            
        } else {
            $assignment = $this->factory->createContainer( $this->getForm() );            
        }
        $this->position_id = "";
        $this->person_id = "";
        


        if ( $this->isPost() ) {
            if ( $assignment instanceof I2CE_Form ) {
                $assignment->load( $this->post );
                $this->person_position = $this->factory->createContainer( $assignment->getParent() );
            }
        } elseif ( $this->get_exists( 'parent' ) ) {
            $this->position_id = $this->get('parent');
            if (strlen($this->position_id) > 0) {
                $this->person_position = $this->factory->createContainer($this->position_id);
            }
        } 
        if ($this->person_position instanceof iHRIS_PersonPosition) {
            $this->person_position->populate();
            $this->person_id = $this->person_position->getParentID();
        } 
        if ( $assignment instanceof I2CE_Form ) {
            $this->setObject( $assignment, I2CE_PageForm::EDIT_PRIMARY );
        }


    }

    /**
     * Load the HTML template files for editing.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
        $this->template->appendFileById( "form_" . $this->getForm( true ) . ".html", "tbody", "person_form" );
    }

    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        if ($this->person_position instanceof iHRIS_PersonPosition) {
            $this->person_position->populate();
            $this->setObject($this->person_position, I2CE_PageForm::EDIT_PARENT);
        }
        $this->person = $this->factory->createContainer( "person".'|'. $this->person_id );
        if ($this->person instanceof iHRIS_Person) {
            $this->person->populate();
        }
        if (!$this->hasPermission('task(person_can_edit_child_form_assignment)')) {
            $this->userMessage("You do not have access to edit the assignment",'notice',false);
            return false;
        }
        parent::loadObjects();

        if ( $this->isPost()) {
             //you are trying to change an existing assignment
            if (!$this->hasPermission('task(person_can_edit_assignment)')) {
                $this->userMessage("You do not have access to edit the assignment",'notice',false);
                return false;
            }
        }
                
    }
        
        
    /**
     * Set the data to be displayed for the page.
     */
    protected function setDisplayData() {
        parent::setDisplayData();
        $this->template->setDisplayData( "person_header", $this->getTitle() );
        $this->template->setDisplayData( "person_form", $this->getForm()  );
    }

    /**
     * Set the I2CE_Form object in the page template.
     * 
     * This method will pass the edit object to the page template so that it can process all the form variables.
     */
    protected function setForm() {
        parent::setForm();
        if ($this->person instanceof iHRIS_Person) {
            $this->template->setForm($this->person);
        }

    }


    /**
     * Run the validation methods for all the objects being edited.
     * 
     * If this is a form submit then run the validation methods for the default object being edited.  The default method
     * calls the {@link I2CE_Form::validate() validate} method on the {@link $edit_obj} object.
     */
    protected function validate() {
        parent::validate();
        if ( $this->isPost() ) {
            //need to check that the start/end dats of the assignement fall in the start/end dates of the person position
            if (!$this->person_position instanceof iHRIS_PersonPosition) {
                I2CE::raiseError("No person position");
                return false;
            }
            $assignment = $this->getPrimary();
            if (!$assignment instanceof Botswana_Assignment) {
                I2CE::raiseError("No assignemnt");
                return false;
            }
            $a_start =  $assignment->start_date;
            if (!$a_start instanceof I2CE_Date) {
                $a_start = false;
            }
            $pp_start =  $this->person_position->start_date;
            if (!$pp_start instanceof I2CE_Date) {
                $pp_start = false;
            }
            $a_end =  $assignment->end_date;
            if (!$a_end instanceof I2CE_Date) {
                $a_end = false;
            }
            $pp_end =  $this->person_position->end_date;
            if (!$pp_end instanceof I2CE_Date) {
                $pp_end = false;
            }
            if ($a_start && $a_end) {
                //first we check start date is less than end date for the assignment
                //if ( $this->getPrimary()->start_date->before( $this->last->start_date ) ) {
                if ($a_end->before($a_start)) { //a start is before a_end.  same is if $a_end->compare($a_start)  > 0
                    $assignment->getField('start_date')->setInvalid("The start date must be after the end date of the work assignement." );
                    return false;
                }
            }
            if ($a_start && $pp_start) {
                if ($a_start->before($pp_start)  ) { //a_start is before p_start
                    $assignment->getField('start_date')->setInvalid("The start date of the assignment must not be be before the start date of the position." );
                    return false;
                } 
            }
            if ($a_end && $pp_end) {
                if ($pp_end->compare($a_end) == -1 ) {  //a_end is after p_end. p_end is before a_end  p_end->compare(a_end) = -1:before, 0:equals
                    $assignment->getField('end_date')->setInvalid("The end date of the assignment must not be after the end date of the position." );
                    return false;
                } 
            }
        }
        return true;
    }



    /**
     * Display the save or confirm buttons as needed.
     * 
     * If the page is a confirmation view then the save / edit button template will be displayed.  
     * Otherwise the confirm and return buttons will be shown.
     * @param boolean $save Flag to show the save button. (Defaults to false)
     * @param boolean $show_edit (defaults to true)
     * @global array
     */
    protected  function displayControls( $save = false, $show_edit = true ) {
        parent::displayControls($save,$show_edit);
        $node = $this->template->getELementById('button_return');
        if ($node instanceof DOMElement) {
            $node->setAttribute('href','view?id=person|' . $this->person_id . '&person_position='); //cheating a bit here.
        }
    }


    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited and return to the view page.  If the action needs to be 
     * logged then the {@link log} method is also called.  Any pages overriding this default save method
     * will need to include any logging necessary.
     */
    protected function save() { 
        $save = true;
        if (!$this->hasPermission('task(person_can_edit_assignment)')) {
            $this->userMessage("You cannot change a person's existing assignment",'notice',true);
            $save = false;
        } 
        if ($save) {
            parent::save();
        }
        $this->setRedirect( "view?id=person|" . $this->person_id );
    }   

}



# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
