<?php
/**
* © Copyright 2011 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package botswana
* @subpackage establishment
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.0.12
* @since v4.0.12
* @filesource 
*/ 
/** 
* Class Botswana_Page_BulkEstablishment
* 
* @access public
*/


class Botswana_Page_BulkEstablishment extends I2CE_PageForm {
    //self::EDIT_SECONDARY



    /**
     * @var string The field, if any, for which we wish to select the list for.
     */
    protected $estab_id = false;
    protected $cs_id = false;
    protected $establishment_period;
    protected $cs_dept;

    
    public function __construct( $args,$request_remainder, $get = null,$post = null) {
        parent::__construct( $args,$request_remainder,$get,$post);
        $this->button_templates = array(
            'button_save'=> 'button_save_export.html',
            'button_save_only'=> 'button_save_export.html',
            'button_save_return'=>'button_save_export.html',
            'button_confirm'=> 'button_save_export.html' ,
            'button_return_only'=> 'button_save_export.html'
            );
    }
    protected function loadObjects() {
        if ($this->request_exists('estab_id')) {
            $this->estab_id = $this->request('estab_id');
        }
        if ($this->estab_id === false)  {
            I2CE::raiseError("No ID Set for establishment period");
            $this->estab_id = false;
            return false;
        }
        $establishment_period   = I2CE_FormFactory::instance()->createContainer($this->estab_id);
        if (!$establishment_period instanceof iHRIS_EstablishmentPeriod) {
            I2CE::raiseError("ID is not valid for establishement");
            $this->estab_id = false;
            return false;
        }
        $establishment_period->populate();
        $this->establishment_period = $establishment_period;
        $this->template->setForm($establishment_period);

        if ($this->request_exists('cs_id')) {
            $this->cs_id = $this->request('cs_id');
        }
        if ($this->cs_id === false) {
            //we will prompt the user for the CS dept
            return true;
        }
        $cs_dept   = I2CE_FormFactory::instance()->createContainer($this->cs_id);
        if (!$cs_dept instanceof Botswana_CS_Department){ 
            I2CE::raiseError("ID is not valid for CS Dept");
            $this->cs_id = false;
            //we will ask for it again
            return true;
        }
        $cs_dept->populate();
        $this->cs_dept = $cs_dept;
        $this->template->setForm($cs_dept);
        if ($this->cs_dept && $this->estab_id && $this->isPost()) {
            $this->loadBulk(false);
        }
    } 

    protected function setDisplayData() {
        parent::setDisplayData();
        if ($this->cs_id !== false) {
            $this->template->setDisplayData('cs_id',$this->cs_id);
        }
        if ($this->estab_id !== false) {
            $this->template->setDisplayData('estab_id',$this->estab_id);
        }
    }

    protected function loadHTMLTemplatesMenu() {
        if (! $this->template->addFile( "bulk_establishment_menu.html" )) {
            return false;
        }
        $cs_depts = I2CE_FormStorage::listFields('cs_dept','name');
        if (!is_array($cs_depts)) {
            I2CE::raiseError("Bad depts");
            return false;
        }
        $css = array();
        foreach ($cs_depts as $id=>$cs_dept) {
            if (!is_array($cs_dept) || !array_key_exists('name',$cs_dept)) {
                continue;
            }
            $css[ 'cs_dept|'.$id ] = $cs_dept['name'];
        }
        $this->template->setDisplayDataImmediate('cs_id',$css);
        return true;
    }

    /*
     *get all the establishments for the specified period and get all the posts.
     * make sure that is an establishment period for each post
     */
    protected function  loadHTMLTemplatesBulk() {
        $this->template->addHeaderLink('mootools-core.js');
        $output = 'html';
        if ($this->request_exists( 'submit_type' ) && $this->request('submit_type') == "export") {
            $output = 'csv';
        }
        return $this->loadBulk($output);
    }


    /*
     *get all the establishments for the specified period and get all the posts.
     * make sure that is an establishment period for each post
     */
    protected function  loadBulk($output = false) {
        if ($output == 'html') {
            $this->template->addFile( "bulk_establishment_base.html" );
            $append_node =  $this->template->getElementByID('establishments');
            if (!$append_node) {
                I2CE::raiseError("Don't know where to append ");
                return false;
            }
            $node = $this->template->loadFile('bulk_establishment_each.html' ,'tr'); 
            if (!$node instanceof DOMNode) {
                I2CE::raiseError("Could not get line");
                return false;
            }
        } else if ($output == 'csv') {
            $out = '';
            $errors = '';
            while  (ob_get_level() > 0) {
                $errors .= ob_get_contents();
                ob_end_clean();
            }
            if ($errors) {
                I2CE::raiseError("Got errors:\n$errors");
            }
            $filename = $this->establishment_period->getField('establishment_type')->getDisplayValue() . ' ' . $this->establishment_period->getField('year')->getDisplayValue();
            header("Content-disposition: attachment; filename=\"$filename\"");
            if (preg_match('/\s+MSIE\s+\d\.\d;/',$_SERVER['HTTP_USER_AGENT'])) {
                header("Content-type: application/vnd.ms-excel");
            } else{
                header("Content-type: text/csv; charset=UTF-8");  
            } 
            flush();             // Flush the headers so the download box appears fast
            $line = array(
                'Title of Post',
                'No. of Posts Requested',
                'Grade',
                'Group',
                'Geographic Location',
                'Brief Justification',
                'Notes',
                'Salary',
                );
            foreach ($line as  &$l) {
                $l = '"' . addslashes($l) . '"';
            } 
            unset($l);
            I2CE::raiseError(implode(",",$line));
            echo implode(",",$line) . "\n"; flush();            
        }
        $no_hidden = array(
            'operator'=>'OR',
            'operand'=>array(
                array(
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'i2ce_hidden',
                    'style'=>'no',
                    'data'=>array( )
                    ),
                array(
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'i2ce_hidden',
                    'style'=>'null',
                    'data'=>array( )
                    )

                )
            );
        $all_post_ids = I2CE_FormStorage::search('post',false,$no_hidden);
        $where = array(
            'operator'=>'FIELD_LIMIT',
            'field'=>'establishment_period',
            'style'=>'equals',
            'data'=>array(
                'value'=>$this->establishment_period->getNameId()
                )
            );
        $where = array (
            'operator'=>'AND',
            'operand'=>array(
                0=>$where,
                1=>$no_hidden
                )
            );
        $post_ids = I2CE_FormStorage::listFields('establishment','post',false,$where);
        foreach ($post_ids as $est_id=> &$post_id) {
            if (!is_array($post_id) || !array_key_exists('post',$post_id)) {
                unset($post_ids[$est_id]);
                continue;
            }
            $post_id = $post_id['post'];
            if ( !is_string($post_id) || ($pos  = strpos($post_id,'|')) === false) {
                unset($post_ids[$est_id]);
                continue;
            }
            $post_id = substr($post_id,$pos + 1);
            if (strlen($post_id)==0) {
                unset($post_ids[$est_id]);
                continue;
            }
            
        }

        $ff = I2CE_FormFactory::instance();
        
        $e_p_id = $this->establishment_period->getID();
        $cs_dept_id = $this->cs_dept->getID();
        $posts = array();
        $grades =array();
        $grades_by_name =array();                    
        foreach ($all_post_ids as $post_id) {
            $estObj = false;
            $sgObj = false;
            $min_salary = 0;
            $postObj = false;
            $est_id = array_search($post_id,$post_ids);
            if ($est_id === false) {
                if ($do_templates) {
                    continue;
                    //we need to create this post.
                    $estObj = $ff->createContainer('etablishment');
                    if (!$estObj instanceof iHRIS_Establishment) {
                        I2CE::raiseError("Could not isntantiate establishemnt");
                        continue;
                    }
                    $estObj->establishment_period = array('establishment', $e_p_id);
                    $estObj->post = array('post', $post_id);
                    $estObj->post = array('cs_dept', $cs_dept_id);
                    $estObj->save($this->user);
                } else {
                    continue;
                }
            } else {
                $estObj = $ff->createContainer(array('establishment',$est_id));
                if (!$estObj instanceof iHRIS_Establishment) {
                    I2CE::raiseError("Could not isntantiate establishemnt for $est_id");
                    continue;
                }      
                $estObj->populate();
                $estObj->load( $this->post,false,false);                
            }
            if ($output) {
                if ($output == 'html') {
                    $e_node = $node->cloneNode(true);
                    $append_node->appendChild($e_node);
                    $this->template->setForm($estObj,$e_node);
                }
                if (!array_key_exists($post_id,$posts)) {
                    $postObj = $ff->createContainer(array('post', $post_id));
                    if ($postObj instanceof I2CE_Form) {
                        $postObj->populate();
                    }
                    $posts[$post_id] = $postObj;
                } else {
                    $postObj = $posts[$post_id];
                }            
                if ($postObj instanceof I2CE_Form) {
                    $sgObj = false;
                    if ($output == 'html') {
                        $this->template->setForm($postObj,$e_node);
                    }
                    if ( ($sgsFields =$postObj->getField('salary_grade')) instanceof I2CE_FormField_MAP_MULT)  {
                        $grades = $sgsFields->getValue();
                        $grade = 'salary_grade|' . $grades[1];
                        $alpha = false;
                        $num = false;
                        $min_grade =false;
                        foreach ($grades as $gradeDetails) {
                            $obj = false;
                            $grade = implode('|', $gradeDetails);
                            if (!array_key_exists($grade, $grades)) {
                                $obj = $ff->createContainer($gradeDetails);
                                if ($obj instanceof iHRIS_SalaryGrade) {
                                    $obj->populate();
                                }
                                $grades[$grade] = $obj;
                            } else {
                                $obj = $grades[$grade];
                            }
                            if (!$obj instanceof I2CE_Form) {
                                continue;
                            }
                            $gradeName = trim($obj->name);
                            if (!$gradeName) {
                                continue;
                            }
                            if (!preg_match('/^\s*([a-zA-Z])([0-9])/',$gradeName,$matches)) {
                                continue;
                            }
                            //we want the lowest alpha and the highest num in that numeric
                            $matches[1] = strtoupper($matches[1]);
                            if ($alpha === false) {
                                $min_grade = $grade;
                                $alpha = $matches[1];
                                $num = $matches[2];
                            } else {
                                if ($matches[1] < $alpha) {
                                    $min_grade = $grade;
                                    $alpha = $matches[1];
                                    $num = $matches[2];
                                } else if ($matches[1] == $alpha && $num < $matches[2]) {
                                    $min_grade = $grade;
                                    $num = $matches[2];
                                }
                            }
                        }
                        if ($min_grade && $alpha && $num) {
                            $grade = $alpha . '|' . $num;

                            if (array_key_exists($min_grade,$grades)) {
                                $sgObj = $grades[$min_grade];
                            } else {
                                $where = array(
                                    'operator'=>'FIELD_LIMIT',
                                    'field'=>'name',
                                    'style'=>'equals',
                                    'data'=>array(
                                        'value'=>$grade
                                        )
                                    );
                                $sg_ids = I2CE_FormStorage::search('salary_grade',false,$where);
                                if (is_array($sg_ids) && count($sg_ids) == 1) {
                                    $sgObj = $ff->createContainer(array('salary_grade',current($sg_ids)));
                                }

                            }
                        }
                    }
                        
                    if ($sgObj instanceof iHRIS_SalaryGrade) {
                        $sal = $sgObj->getField('start');
                        if  ($sal instanceof iHRIS_FormField_Currency) {
                            $sval =  $sal->getValue();
                            if (is_array($sval)) {
                                $min_salary = max(0,$sval[1]);
                            }
                        }
                    }
                    if ($output == 'html') {
                        $this->template->setDisplayDataImmediate('salary',$min_salary,$e_node);
                    }
                }
            }
            if ($output == 'html') {
                $this->setObject($estObj,I2CE_PageForm::EDIT_SECONDARY,$e_node);
            } else if ($output == 'csv') {
                $line = array();
                if ($postObj instanceof I2CE_Form) {
                    $line[0] = $postObj->getField('name')->getDisplayValue() ;
                    $line[2] = $postObj->getField('salary_grade')->getDisplayValue() ;
                    $line[3] = $postObj->getField('post_group')->getDisplayValue() ;
                } else {
                    $line[0] = '';
                    $line[2] = '';
                    $line[3] = '';
                }
                if ($estObj instanceof I2CE_Form) {
                    $line[1] = $estObj->getField('amount')->getDisplayValue();
                    $line[4] = $estObj->getField('location')->getDisplayValue();
                    $line[5] = $estObj->getField('justification')->getDisplayValue();
                    $line[6] = $estObj->getField('notes')->getDisplayValue();
                } else {
                    $line[1] = '';
                    $line[4] = '';
                    $line[5] = '';
                    $line[6] = '';
                }
                $line[7] = $min_salary;
                ksort($line);
                foreach ($line as  &$l) {
                    $l = '"' . addslashes($l) . '"';
                } 
                unset($l);
                I2CE::raiseError(implode(",",$line));
                echo implode(",",$line) . "\n"; flush();
            } else {
                $this->setObject($estObj,I2CE_PageForm::EDIT_SECONDARY);
            }
        }
        if ($output == 'csv') {
            exit();
        }
        return true;
        
    }



        protected function loadHTMLTemplates() {
        if ($this->cs_id === false) {
            return $this->loadHTMLTemplatesMenu();
        } else if ($this->estab_id !== false ) {
            return $this->loadHTMLTemplatesBulk();
        } else {
            return false;
        }
    }

        
    /**
     * Display the save or confirm button templates as needed.
     * @param boolean $save Flag to show the save button.
     * @param boolean $show_edit (defaults to true)
     */
    protected function displayControls( $save = false, $show_edit = true) {
        if ($this->cs_id === false) {
            return ;
        }
        parent::displayControls($save,$show_edit);
    }



    
    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited b
     * @global array
     */
    protected function save() {
        foreach( $this->objects[ I2CE_PageForm::EDIT_SECONDARY] as $obj ) {
            if (!$obj->save( $this->user )) {
                I2CE::raiseError("Not saved");
                return false;
            }
        }
    
        if ($this->post_exists('redirect')) {
            $redirect = $this->post('redirect');
        } else  {
            $redirect = 'bulkEstablishment?estab_id=' . $this->establishment_period->getNameId() . '&cs_id=' . $this->cs_dept->getNameID();
        }
        $this->setRedirect($redirect );
        return true;
    }



    

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
