<?php
/**
* © Copyright 2012 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package Botswana
* @subpackage 
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.1.4.1
* @since v4.1.4.1
* @filesource 
*/ 
/** 
* Class Botswana_Modules_Training
* 
* @access public
*/


class Botswana_Module_Training  extends I2CE_Module {
	
    public static function getMethods() {
      return array(
       'iHRIS_PageView->action_person_scheduled_training_course' => array('priority'=>300,'method'=>'action_person_scheduled_training_course'),
                   );
    }

    public function action_person_scheduled_training_course($page) {
        if (!$page instanceof iHRIS_PageView) {
            return false;
        }
        $template = $page->getTemplate();
        $appendNode = $template->getElementById('person_scheduled_training_course');
        if (!$appendNode instanceof DOMNode) {
            return true;
        }
        $person = $page->getPerson();
        if (!$person instanceof iHRIS_Person) {
            return false;
        }
        $factory = I2CE_FormFactory::instance();
        $trainingIds = $this->getEnrolledCourseIds($person);
        $pstcs = array();
        foreach ($trainingIds as $trainingId) {
            $trainingForm = $factory->createContainer('person_scheduled_training_course'.'|'.$trainingId);
            if (!$trainingForm instanceof iHRIS_Person_Scheduled_Training_Course) {
                continue;
            }
            $trainingForm->populate();            
            $pstcs[] = $trainingForm;
        }
        if (count($pstcs) == 0) {
            return true;
        }
        $do_exam = I2CE_ModuleFactory::instance()->isEnabled("training-exam");
        foreach ($pstcs as $child) {
            $node = $template->appendFileByNode('view_person_scheduled_training_course.html', 'div',  $appendNode );
            if (!$node instanceof DOMNode) {
                I2CE::raiseError("Could not find template $template for child form $form of person");
                return false;
            }
            $template->setForm($child,$node);
            if ($do_exam) {
                $child->populateChildren('training_course_exam',array('evaluation_date'));
                $exams = $child->getChildren('training_course_exam');
                
                if (count($exams) > 0) {
                    $template->setDisplayDataImmediate('has_exam_results',1,$node);
                    foreach ($exams as $exam) {
							$scheduled_course = $factory->createContainer($child->scheduled_training_course);
							$scheduled_course->populate();
							$assessment=$scheduled_course->assessment[1];
							if($assessment=="evaluation")
							$exam_template="view_training_course_evaluation.html";
							else if($assessment=="exam" or $assessment=="")
							$exam_template="view_training_course_exam.html";
                        if (!  ($examNode = $template->appendFileById($exam_template,'tbody','exam_results',false,$node)) instanceof DOMNode) {
                            continue;
                        }
                        $template->setForm($exam,$examNode);
                    }
                } else {
                    $template->setDisplayDataImmediate('has_exam_results',0,$node);
                }
            }
            
            $scheduled_course = $factory->createContainer($child->scheduled_training_course);
            if (!$scheduled_course instanceof iHRIS_Scheduled_Training_Course || $scheduled_course->getId() =='0') {
                I2CE::raiseError( "Bad Scheduled Training Course:" . $child->scheduled_training_course );
                continue;
            }
            $scheduled_course->populate();
            $template->setForm($scheduled_course,$node);
            $course = $factory->createContainer($scheduled_course->training_course);
            if (!$course instanceof iHRIS_Training_Course || $course->getId() == '0') {
                I2CE::raiseError( "Bad Couse:" . $scheduled_course->getParent());
                continue;
            }
            $course->populate();
            $template->setForm($course,$node);
            
        }

        return true;
    }
    
        protected function getEnrolledCourseIDs($personForm) {
        if (!$personForm instanceof iHRIS_Person || $personForm->getId() == '0') {
            //echo "A";
            return array();
        }
        $where = array(
            'operator' => 'OR',
            'operand'=>array(
                0=>array(
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'attending',
                    'style'=>'equals',
                    'data'=>array(
                        'value'=>1
                        )),
                1=>array(
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'attending',
                    'style'=>'null',
                    'data'=>array()
                    )
                )
            );
        return $personForm->getChildIds('person_scheduled_training_course','-request_date',$where); //get all trainings.

    }
    public static function getHooks() {
      return array(
        'validate_form_scheduled_training_course' => 'validate_form_scheduled_training_course',
                   );
    }
 
    /**
     * Validate the $form form.
     * @param I2CE_Form $form
     */
    public function validate_form_scheduled_training_course( $form ) {
      
      $start_date = $form->start_date->getValues();
      $end_date = $form->end_date->getValues();
      $training_course=$form->training_course[0]."|".$form->training_course[1];   
      $where=array(
            'operator'=>'AND',
            'operand'=>array(
              0=>array('operator'=>'FIELD_LIMIT',
								 'field'=>'start_date',
								 'style'=>'equals',
								 'data'=>array('value'=>$start_date['year'].'-'.$start_date['month'].'-'.$start_date['day'].' 00:00:00')),
							1=>array('operator'=>'FIELD_LIMIT',
								 'field'=>'end_date',
								 'style'=>'equals',
								 'data'=>array('value'=>$end_date['year'].'-'.$end_date['month'].'-'.$end_date['day'].' 00:00:00')),
              2=>array('operator'=>'FIELD_LIMIT',
                 'field'=>'venue',
                 'style'=>'equals',
                 'data'=>array('value'=>$form->venue)),
              3=>array('operator'=>'FIELD_LIMIT',
                 'field'=>'training_course',
                 'style'=>'equals',
                 'data'=>array('value'=>$training_course)),
							));
      $existing_scheduled_course=I2CE_FormStorage::search("scheduled_training_course",false,$where);
      if(count($existing_scheduled_course)>0 && $form->getID()==0){
          $form->setInvalidMessage("training_course","This course already scheduled at ".$form->venue." on these dates");
        }
      else if($form->getID()!=0 && count($existing_scheduled_course)>0 && $existing_scheduled_course[0]!=$form->getID()){
          $form->setInvalidMessage("training_course","This course already scheduled at ".$form->venue." on these dates");
        }
     }
    
    /**
     * Upgrade module method
     */
    public function upgrade($old_vers,$new_vers) {
        if (I2CE_Validate::checkVersion($old_vers,'<','4.1.1.22')) {
            // if (! $this->moveFormFromEntryToMagic('training_course')) {
            //     return false;
            // }
        }
        return true;
    }


    protected function moveFormFromEntryToMagic($form) {
	if ( ! ($magic_storage = I2CE_FormStorage::getMechanismByStorage('magicdata')) instanceof I2CE_FormStorage_Mechanism) { 
	    I2CE::raiseError("Could not instantiate magicdata form storage");
	    return false;
	}
	if ( ! ($entry_storage = I2CE_FormStorage::getMechanismByStorage('entry')) instanceof I2CE_FormStorage_Mechanism) { 
	    I2CE::raiseError("Could not instantiate entry form storage");
	    return false;
	}
	$user = new I2CE_User();
	if (! I2CE_FormStorage::moveData($form,$entry_storage,$magic_storage,$user)) {
	    I2CE::raiseError("Could not export dpsm_dept to magicdata");
	    return false;
	}
	I2CE::getConfig()->modules->forms->forms->$form->storage = "magicdata";
	return true;
    }




}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
