<?php
/**
* © Copyright 2008 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
*  iHRIS_Scheduled_Training_Course
* @package I2CE
* @subpackage Core
* @author Ally Shaban <allyshaban5@yahoo.com>
* @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
* This file is part of I2CE. I2CE is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free Software Foundation; either 
* version 3 of the License, or (at your option) any later version. I2CE is distributed in the hope 
* that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
* or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have 
* received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
* @version 2.1
* @access public
*/


class Botswana_Scheduled_Training_Course extends iHRIS_Scheduled_Training_Course{    
    public function enrolledStudentsListCompact($node,$template,$readonly=false) {
        if ( ! ($main_node =$template->appendFileByNode('training_course_exam_base_compact.html','div',$node )) instanceof DOMNode ) {
            return false;
        }
        if (! ($list_node = $template->getElementById('student_exams',$main_node)) instanceof DOMNode) {
            return false;
        }
        $node->appendChild($list_node);       
        $list = $this->getEnrolledStudents();
        $template->addHeaderLink("mootools-core.js");
        $template->addHeaderLink("mootools-more.js");
        $template->addHeaderLink("editstudent.js");
        $has_exam = I2CE_ModuleFactory::instance()->isEnabled('training-exam'); 
        if (!$has_exam) {
            return;
        }
        $where_exam =  
            array('operator' => 'OR',
            'operand'=>array(
                array(
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'i2ce_hidden',
                    'style'=>'no'),
                array(
                    'operator'=>'FIELD_LIMIT',
                    'field'=>'i2ce_hidden',
                    'style'=>'null')
                ));
                
        $exams = I2CE_FormStorage::listDisplayFields('training_course_exam_type',array('name'),false,$where_exam,array('name'));
        $passing_score = '';
        $passing_scores = I2CE_FormStorage::lookupField('training_course',$this->getField('training_course')->getMappedID(), array('passing_score'),false);
        if (count($passing_scores) == 1) {
            $passing_score = current($passing_scores);
            if (!is_numeric($passing_score) || $passing_score < 0 || $passing_score > 100) {
                $passing_score = '';
            }
        }
        
        $stc_dup_where = array(
            'operator'=>'FIELD_LIMIT',
            'field'=>'training_course',
            'style'=>'equals',
            'data'=>array(
                'value'=> $this->getField('training_course')->getDBValue()
                ));

        
        $dup_where = array(); //this will be a search on person_scheduled_training_course
        foreach (I2CE_FormStorage::search('scheduled_training_course',false,$stc_dup_where) as $stc) {
            if ($stc == $this->getID()) {
                //don't include this course when looking for duplicate scheuling.
                continue;
            }
            $dup_where[] = 
                array(
                    'operator'=>'AND',
                    'operand'=>array(
                        0=> array(
                            'operator'=>'FIELD_LIMIT',
                            'field'=>'scheduled_training_course',
                            'style'=>'equals',
                            'data'=>array(
                                'value'=>'scheduled_training_course|' .$stc
                                )),
                        1=>array(
                            'operator' => 'OR',
                            'operand'=>array(
                                0=>array(
                                    'operator'=>'FIELD_LIMIT',
                                    'field'=>'attending',
                                    'style'=>'equals',
                                    'data'=>array(
                                        'value'=>1
                                        )),
                                1=>array(
                                    'operator'=>'FIELD_LIMIT',
                                    'field'=>'attending',
                                    'style'=>'null',
                                    'data'=>array()
                                    )
                                )
                            )
                        )
                    );
        }
        foreach ($exams as $i=>$data)  {
            if (!is_array($data) || !array_key_exists('name',$data) || !$data['name']) {
                unset($exams[$i]);
                continue;
            }
        }
        
        $num_exams = count($exams);
        if ($num_exams > 0) {
            $style = "display:table-cell;width:"  . (66/$num_exams) .   '%';
        }else {
            $style = "display:table-cell";
        }
        
        //check assessment type
        list($stc_form,$stc_id)=array_pad(explode("|",$this->getNameId(),2),2,"");
        $assessment=I2CE_FormStorage::lookupField($stc_form,$stc_id,array("assessment"),false);
        $assessment=$assessment["assessment"];
        if (($header_node = $template->getElementByID("header_row")) instanceof DOMNode) {
        	
          if($assessment=="assessment|exam" or $assessment==""){ //assessment is based on an exam
            foreach ($exams as $id=>$data) {
                $exam_title_node = $template->createElement("div",array('style'=>$style),$data['name']);
                $template->appendNode($exam_title_node,$header_node);
            }
          }
          
          else if($assessment=="assessment|evaluation") //assessment is based on pass/fail without an exam
          {
                $exam_title_node = $template->createElement("div",array('style'=>$style),"Evaluation");
                $template->appendNode($exam_title_node,$header_node);
                $exams=array();
                $exams=array("final"=>"final");     	
          }
        }
        foreach ($list as $id=>$sdata) {
            $studentNode = $template->appendFileByNode('enrolled_students_compact.html','div',$list_node);
            if (!$studentNode instanceof DOMNode) {
                return false;
            }
            $studId = 'student_' . $id;
            $studentNode->setAttribute('id',$studId);
            $template->setDisplayDataImmediate('id','id=' . $id, $studentNode);
            $template->setDisplayDataImmediate('firstname',$sdata['firstname']  , $studentNode);
            $template->setDisplayDataImmediate('surname',$sdata['surname'], $studentNode);
				
				if($assessment=="assessment|exam" or $assessment=="") //assessment is by exam
            $scores = I2CE_FormStorage::listFields('training_course_exam',array('score','training_course_exam_type', 'retake'),$sdata['person_scheduled_training_course']);
            if($assessment=="assessment|evaluation") //assessment is by panel decision
            $scores = I2CE_FormStorage::listFields('training_course_exam',array('evaluation','training_course_exam_type'),$sdata['person_scheduled_training_course']);
           
            foreach ($exams as $examid=>$data) {
                $examsNode = $template->createElement("div",array('style'=>$style));
                $template->appendNode($examsNode,$studentNode);
                
                $template->setDisplayData('passing_score',$passing_score,$examsNode);

                $score = false;
                
                $exam_id ='0';

                foreach ($scores as $sid=>$data) {
                    if (!is_array($data)
                        || !array_key_exists('training_course_exam_type',$data)
                        || $data['training_course_exam_type'] != "training_course_exam_type|" . $examid
                        )
                    {
                    	echo $score;
                        continue;
                    }                             
                    if($assessment=="assessment|exam" || $assessment=="") {
                      if($data['retake'] == 1){
                        $score =$data['score'].'*';
                      }else{
                        $score = $data['score'];
                        }
                    }
                    
                    if($assessment=="assessment|evaluation")
                    $score =$data['evaluation'];
                    
                    //break;
                }                
                

                if($assessment=="assessment|exam" or $assessment=="")
                {
                $template->appendFileByNode('student_exam.html','div',$examsNode );
                $finalNode = $template->getElementByName('final_exam',0,$examsNode);
                if (!$finalNode) {
                    continue;
                }
                $template->setDisplayDataImmediate('final_exam',$score,$finalNode);
                if ($passing_score) {
                    if ($score >= $passing_score) {
                        $finalNode->setAttribute('style','color:green');
                    } else {
                        $finalNode->setAttribute('style','color:red');
                    }
                }
                if (!$passing_score) {
                    $passing_score = 0;
                }
                $js = "changeFinalExamGrade(this,'".addslashes($examid)."','" .  addslashes($sdata['person_scheduled_training_course'])  .  "','" . addslashes($exam_id) . "'," . $passing_score . ",'assessment|exam');";
                $finalNode->setAttribute('onchange',$js);
					}
					
					elseif ($assessment=="assessment|evaluation"){ //assessment is by panel decision, so load the form for Pass/Fail
					$template->appendFileByNode('student_evaluation.html','div',$examsNode );
                $finalNode = $template->getElementByName('evaluation',0,$examsNode);
                $option=$template->createElement("option",array("value"=>"sel","name"=>"evaluation"),"Select Evaluation");
                $template->appendNode($option,$finalNode);
                $option=$template->createElement("option",array("value"=>"evaluation|pass"),"Pass");
                $template->appendNode($option,$finalNode);
                if($score=="evaluation|pass")
                $option->setAttribute("selected","selected");                
                $option=$template->createElement("option",array("value"=>"evaluation|fail"),"Fail");
                $template->appendNode($option,$finalNode);
                
                if($score=="evaluation|fail")
                $option->setAttribute("selected","selected");                
                if (!$finalNode) {
                    continue;
                }

                    if ($score == "evaluation|pass") {
                        $finalNode->setAttribute('style','color:green');
                    } else {
                        $finalNode->setAttribute('style','color:red');
                    }

                $js = "changeFinalExamGrade(this,'".addslashes($examid)."','" .  addslashes($sdata['person_scheduled_training_course'])  .  "','" . addslashes($exam_id) . "'," . $passing_score . ",'assessment|evaluation');";
                $finalNode->setAttribute('onchange',$js);	
                if ($readonly) {
                    $finalNode->setAttribute('readonly','readonly');
                }

                    }
					
            }
            $clear_node = $template->createElement("hr",array("style"=>"clear: both; max-height:0px; padding:0px; margin:0px;"));
            $template->appendNode($clear_node,$list_node);

        }
    }
}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
