<?php
/*
 * © Copyright 2012 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Edit participants action for a training
 * @package iHRIS
 * @subpackage Common
 * @access public
 * @author  Carl Leitner <litlfred@ibiblio.org> 
 * @copyright Copyright &copy; 2012 IntraHealth International, Inc. 
 * @since v4.1.3
 * @version v4.1.3
 */

/**
 * The action page class for editing particpants for a training
 * @package iHRIS
 * @subpackage Train
 * @access public
 */
class iHRIS_PageActionStudentLettersPass extends I2CE_PageGenerateRelationshipTemplate {

    protected $person_id = array();

    protected function loadPrimary() {
    if($this->request_exists('id'))
    	{
    		$error_msg="No Student Passed This Course";
    		$redirection="view_list?mapped=&type=scheduled_training_course&id=".$this->request("id");
    	$where=array("operator"=>"AND","operand"=>array(
    					0=>array(
    								"operator"=>"FIELD_LIMIT",
    								"field"=>"scheduled_training_course",
    								"style"=>"equals",
    								"data"=>array("value"=>$this->request("id"))
    								),
    					1=>array(
    								"operator"=>"FIELD_LIMIT",
    								"field"=>"training_course_evaluation",
    								"style"=>"equals",
    								"data"=>array("value"=>"training_course_evaluation|pass")
    								),
    					 ));
    					 
    	$persons=I2CE_FormStorage::listFields("person_scheduled_training_course",array("parent"),true,$where);

      foreach ($persons as $person_id)
      $this->person_id[] = $person_id["parent"];
      $stc_id=$this->request("id");
    	}
      else if ($this->request_exists('student_id')
            && in_array('person',$this->formRelationship->getFormNames())
            && ($student_id = $this->request('student_id'))
            && ($pos = strpos($student_id,'|')) !== false
            && ($id = substr($student_id,$pos + 1)) != ''
            && ($stc_id = I2CE_FormStorage::lookupField('person_scheduled_training_course',$id,array('scheduled_training_course'),'')) != ''
            && ($person_id = I2CE_FormStorage::lookupField('person_scheduled_training_course',$id,array('parent'),'')) != ''
            ) {      	
			    	//check if this person passed this course
			    	$evaluation = I2CE_FormStorage::lookupField('person_scheduled_training_course',$id,array('training_course_evaluation'),'');
			    	if($evaluation=="training_course_evaluation|pass")
			    	$this->person_id[] = $person_id;
			    	$error_msg="Student didnt pass this course";
			    	$redirection="view?id=".$person_id;
        }
    
    	if(count($this->person_id)==0)
    	{
   		$this->userMessage($error_msg);
			$this->setRedirect($redirection);
			return false;
    	}
    	else
    	{
            $formFactory = I2CE_FormFactory::instance();
            if  (! ($this->primObj = $formFactory->createContainer($stc_id)) instanceof I2CE_Form
                 || $this->formRelationship->getPrimaryForm() != $this->primObj->getName()
                ) {
                I2CE::raiseError("invalid form id :" . print_r($this->request(),true) . "\ndoes not match " . $this->formRelationship->getPrimaryForm());
                return false;
            }
            $this->primObj->populate();
       }    
    }
        
    protected $trainingCoursePassMarks = array(
          'training_course|225935' => 80, //accf-doctors
          'training_course|KITSO3' => 75, //accf-nurses
          'training_course|225932' => 75, //refresher-nurses
          'training_course|225937' => 80, //refresher-doctors
          'training_course|KITSO2' => 75 //mac
          //'training_course|KITSO4' => 75, //iacc
          //'training_course|KITSO45' => 75, //npd
          //'training_course|KITSO44' => 75 //nd
        );
    
    protected function getTrainingCoursePassMark(){
      $ff = I2CE_FormFactory::instance();
        if($this->request_exists('id')){
            $stCourses = $this->request('id');
            $stCoursesObj = $ff->createContainer($stCourses);
            $stCoursesObj->populate();
            $trCourse = $stCoursesObj->getField('training_course')->getDBValue();
            if(array_key_exists($trCourse, $this->trainingCoursePassMarks)){
                return true;
              }
            else{
              return false;
            }
          }
      }


    /**
     *Loads in the requeted data from the relationship
     * @returns boolean  True on success
     */
    protected function loadData($as_iterator = true) {
    	if(count($this->person_id)>0)
    	{
            $p_where = array(
                'operator'=>'FIELD_LIMIT',
                'field'=>'parent',
                'style'=>'in',
                'data'=>array(
                    'value'=>$this->person_id
                    )
                );
            $this->formRelationship->setAdditionalLimit('person_scheduled_training_course',$p_where);
		return parent::loadData($as_iterator);
		}
    }


    protected function getODTTemplate() {
    	if(count($this->person_id)>0)
    	{
        //first we see if we have a letter uploaded to the training course itself
        $have_certificate =false;
        if (($tcField = $this->primObj->getField('training_course')) instanceof I2CE_FormField_MAP
            &&  ($tcObj = $tcField->getMappedFormObject()) instanceof iHRIS_Training_Course) {
            $tcObj->populate();
            if ( ($docField = $tcObj->getField('ack_letter')) instanceof I2CE_FormField_DOCUMENT
                 && $docField->isValid()
                 && ($content = $docField->getValue())
                 && ($name = $docField->getFileName())) {
                $pos = strrpos($name,'.');
                $ext ='';
                if ($pos !== false) {
                    $ext = substr($name,$pos);
                    $name = substr($name, 0,$pos);
                }
                $this->template_file = tempnam(sys_get_temp_dir(), basename($name .'_' ))  . $ext;
                I2CE::raiseError($this->template_file);
                file_put_contents($this->template_file,$content);            
                return true;
            }
        }
        return parent::getODTTemplate();
      }
    }

    
}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
